<?php

namespace SGalinski\SgNews\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Extbase\Domain\Model\FrontendUser;
use TYPO3\CMS\Extbase\Domain\Model\FileReference;
use TYPO3\CMS\Extbase\DomainObject\AbstractEntity;
use TYPO3\CMS\Extbase\Persistence\Generic\LazyLoadingProxy;

/**
 * Author
 */
class Author extends AbstractEntity {
	/**
	 * @var string
	 */
	protected $name = '';

	/**
	 * @var string
	 */
	protected $email = '';

	/**
	 * @var string
	 */
	protected $description = '';

	/**
	 * @lazy
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FrontendUser
	 */
	protected $frontendUser;

	/**
	 * @lazy
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 */
	protected $image;

	/**
	 * @return string
	 */
	public function getName(): string {
		return $this->name;
	}

	/**
	 * @param string $name
	 */
	public function setName(string $name): void {
		$this->name = $name;
	}

	/**
	 * @return string
	 */
	public function getNameAndRespectFrontendUser(): string {
		$frontendUser = $this->getFrontendUser();
		if ($frontendUser) {
			$name = $frontendUser->getName();
			if (!empty($name)) {
				return $name;
			}
		}

		return $this->name;
	}

	/**
	 * @return string
	 */
	public function getEmail(): string {
		return $this->email;
	}

	/**
	 * @param string $email
	 */
	public function setEmail(string $email): void {
		$this->email = $email;
	}

	/**
	 * @return string
	 */
	public function getEmailAndRespectFrontendUser(): string {
		$frontendUser = $this->getFrontendUser();
		if ($frontendUser) {
			$email = $frontendUser->getEmail();
			if (!empty($email)) {
				return $email;
			}
		}

		return $this->email;
	}

	/**
	 * @return string
	 */
	public function getDescription(): string {
		return $this->description;
	}

	/**
	 * @param string $description
	 */
	public function setDescription(string $description): void {
		$this->description = $description;
	}

	/**
	 * @return FrontendUser
	 */
	public function getFrontendUser(): ?FrontendUser {
		if ($this->frontendUser instanceof LazyLoadingProxy) {
			$this->frontendUser->_loadRealInstance();
		}

		return $this->frontendUser;
	}

	/**
	 * @param FrontendUser $frontendUser
	 */
	public function setFrontendUser(FrontendUser $frontendUser): void {
		$this->frontendUser = $frontendUser;
	}

	/**
	 * @return FileReference
	 */
	public function getImage(): ?FileReference {
		if ($this->image instanceof LazyLoadingProxy) {
			$this->image->_loadRealInstance();
		}

		return $this->image;
	}

	/**
	 * @param FileReference $image
	 */
	public function setImage(FileReference $image): void {
		$this->image = $image;
	}

	/**
	 * @return string
	 */
	public function getImageAndRespectFrontendUser(): string {
		$frontendUser = $this->getFrontendUser();
		if ($frontendUser) {
			$images = $frontendUser->getImage();
			if ($images->count() > 0) {
				return $images->current();
			}
		}

		if ($this->image instanceof LazyLoadingProxy) {
			$this->image->_loadRealInstance();
		}

		return $this->image;
	}
}
